import java.sql.*;
public class JDBCExamples {
	private Connection con;

	public static void main(String [] args){
		try {
			JDBCExamples j=new JDBCExamples();
			j.dropTables();
			j.createTables();
			j.addRows();
			j.runQueries();
			j.runUpdates();
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		}
	}

	JDBCExamples(){
		try { 
			// Load the JDBC-ODBC bridge driver
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver"); 
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: "); 
			System.err.println(e.getMessage());
		}
		try {
			// Should have a odbc datasource with the name supplier-part
			String url = "jdbc:odbc:supplier-part";

			// This database requires no userid/password to login
			con = DriverManager.getConnection(url, "", "");
		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		}
	}

	void createTables()throws SQLException{
		// Get a statement (used to issue SQL statements to the DB)
		Statement stmt = con.createStatement();
		// Create all tables
		stmt.executeUpdate("CREATE TABLE PART (PNO INTEGER, PNAME VARCHAR(20), PRICE NUMBER);");
		stmt.executeUpdate("CREATE TABLE SELLS (SNO INTEGER, PNO INTEGER); ");
		stmt.executeUpdate("CREATE TABLE SUPPLIER (SNO INTEGER, SNAME VARCHAR(20), CITY VARCHAR(20));");
		stmt.executeUpdate("CREATE TABLE SALES (weekNo INTEGER, PNO INTEGER, SNO INTEGER, AMOUNT INTEGER);");
		stmt.executeUpdate("CREATE TABLE salesByPart (weekNo INTEGER, PNO INTEGER, AMOUNT INTEGER);");
		stmt.close();
	}

	// Drop all tables
	void dropTables() throws SQLException {
		dropTable("PART"); 
		dropTable("SELLS"); 
		dropTable("SUPPLIER");
	}

	void dropTable(String tableName) throws SQLException {
		// Get a statement (used to issue SQL statements to the DB)
		Statement stmt = con.createStatement();

		// We handle the non-existence of a table in a poor way,
		// we just catch the exception.
		// For a better way to deal with this, see java.sql.DatabaseMetaData
		try {
		   	stmt.executeUpdate("DROP TABLE "+tableName);
		} catch (SQLException ex) { }
		stmt.close();
	}

	// Pre: Tables PART, SELLS and SUPPLIER exist and are 
	//	empty in the database con. Tables are properly defined.
	// Post: All tables are filled with initial values (vague post condition)
	void addRows() throws SQLException {
		//PART 
		// using a simple query string
		Statement stmt = con.createStatement();
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (1,'Screw',10);");
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (2,'Nut',8);");

		// building a query string
		int pno; String pname; int price;
		String insertPart;
		pno=3; pname="Bolt"; price=15;
		insertPart="INSERT INTO PART (PNO, PNAME, PRICE ) VALUES ("+pno+",'"+pname+"',"+price+");";
		stmt.executeUpdate(insertPart);

		pno=4; pname="Cam"; price=25;
		insertPart="INSERT INTO PART (PNO, PNAME, PRICE ) VALUES ("+pno+",'"+pname+"',"+price+");";
		stmt.executeUpdate(insertPart);
		stmt.close();

		// SELLS (using a prepared statement)
		// This reduces the overhead of parsing the statement
		// and constructing the SQL string
		String insertSells="INSERT INTO SELLS (SNO, PNO) VALUES (?,?);";
		PreparedStatement ps = con.prepareStatement(insertSells);
		Sells sells[]={
			new Sells(1, 1), new Sells(1, 2), new Sells(2, 4),
			new Sells(3, 1), new Sells(3, 3), new Sells(4, 2),
			new Sells(4, 3), new Sells(4, 4),
		};
		for(int i=0;i<sells.length;i++){
			ps.setInt(1,sells[i].getSNO()); // Fill in the first ?
			ps.setInt(2,sells[i].getPNO()); // Fill in the second ?
			ps.executeUpdate();
		}
		ps.close();

		//SUPPLIER 
		// Each Supplier knows how to write themselves to the DB)
		Supplier supplier;
		supplier=new Supplier(1,"Smith","London"); supplier.write(con);
		supplier=new Supplier(2,"Jones","Paris"); supplier.write(con);
		supplier=new Supplier(3,"Adams","Vienna"); supplier.write(con);
		supplier=new Supplier(4,"Blake","Rome"); supplier.write(con);

	}

	void runQueries() throws SQLException {
		String query=null;
		Statement stmt=con.createStatement();

		// A ResultSet is returned when we execute a search query. 
		// This allows us to scroll through the rows of the result 
		// table one row at a time
		ResultSet rs=null;

		query="SELECT * FROM PART;";
		System.out.println(query);

		// Send the query to the DB
		rs = stmt.executeQuery(query);

		// Go through all rows
		while(rs.next()){

			// Pull out individual columns from the current row
			int pno=rs.getInt("PNO");
			String pname=rs.getString("PNAME");
			double price=rs.getDouble("PRICE");

			// Print out the values
			System.out.println(pno+"\t"+pname+"\t"+price);
		} 
		rs.close();

		//query="SELECT * FROM PART WHERE PRICE > 10;";
		//query="SELECT PNAME, PRICE FROM PART WHERE PRICE > 10;";
		//query="SELECT PNAME, PRICE FROM PART WHERE PNAME = 'Bolt' AND (PRICE = 0 OR PRICE <= 15);";
		//query="SELECT * FROM SUPPLIER, PART; ";

		query="SELECT * FROM SUPPLIER S1, SUPPLIER S2; "; 
		printMetaData(query);
		// Can't pull columns out of above query. 
		// MSAccess Driver does not rename columns
		// so we won't be able to identify resulting columns.
		
		// SQL Allows us to rename columns.
		query="SELECT "
			+" S1.SNO as S1SNO, S1.SNAME as S1SNAME, S1.CITY as S1CITY"
			+",S2.SNO as S2SNO, S2.SNAME as S2SNAME, S2.CITY as S2CITY"
			+" FROM SUPPLIER S1, SUPPLIER S2;";
		System.out.println(query);
		rs = stmt.executeQuery(query);
		printMetaData(rs);
		while(rs.next()){
			int s1Sno=rs.getInt("S1SNO");
			String s1Sname=rs.getString("S1SNAME");
			String s1City=rs.getString("S1CITY");
			int s2Sno=rs.getInt("S2SNO");
			String s2Sname=rs.getString("S2SNAME");
			String s2City=rs.getString("S2CITY");
			System.out.print(s1Sno+" "+s1Sname+" "+s1City+" ");
			System.out.println(s2Sno+" "+s2Sname+" "+s2City);
		} 
		rs.close();

		// A join 
		query="SELECT S.SNAME, P.PNAME "
			+" FROM SUPPLIER AS S, PART AS P, SELLS AS SE "
			+" WHERE S.SNO = SE.SNO AND P.PNO = SE.PNO;";
		System.out.println(query);
		rs = stmt.executeQuery(query);
		while(rs.next()){
			String sname=rs.getString("SNAME");
			String pname=rs.getString("PNAME");
			System.out.println(sname+" "+pname);
		} 
		rs.close();

		/* A join with order by
		query="SELECT S.SNAME, P.PNAME "
    			+" FROM SUPPLIER S, PART P, SELLS SE "
    			+" WHERE S.SNO = SE.SNO AND P.PNO = SE.PNO "
    			+" ORDER BY S.SNAME;";
		*/

		stmt.close();

	}

	void runUpdates() throws SQLException {
		runUpdate("INSERT INTO SUPPLIER ( SNO, SNAME, CITY ) SELECT 1, 'Smith', 'London';");
		runUpdate("UPDATE PART SET PRICE = 15 WHERE PNAME = 'Screw';");
		runUpdate("DELETE * FROM SUPPLIER WHERE SNAME = 'Smith';");
	}

	void runUpdate(String query) throws SQLException{
		System.out.println(query);
		Statement stmt=con.createStatement();
		int r=stmt.executeUpdate(query);
		System.out.println(r+" rows effected");
		stmt.close();
	}

	void updatesWithTransaction() throws SQLException {
		Statement stmt=con.createStatement();
		// Normally, each executeUpdate affects the database.
		// We want to execute a collection of updates all together
		// so that they all effect the database or none do.

		con.setAutoCommit(false); // Change to transactional mode.
			// now updates are reflected in the database only
			// after a commit()
		
		// transaction begins here
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (10,'Hammer',10.23);");
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (20,'Screw Driver',11.55);");
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (30,'Saw',19.21);");
		con.commit(); // transaction ends here

		// transaction begins here
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (10,'Hammer',10.23);");
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (20,'Screw Driver',11.55);");
		stmt.executeUpdate("INSERT INTO PART (PNO, PNAME, PRICE ) VALUES (30,'Saw',19.21);");
		con.commit(); // transaction ends here


		stmt.close();
	}

	// Print some of a ResultsSets meta data
	void printMetaData (ResultSet rs)throws SQLException {
		System.out.println("ResultSet meta data follows");
		ResultSetMetaData rsmd=rs.getMetaData();
		int columnCount=rsmd.getColumnCount();
		for(int i=1;i<=columnCount;i++){
			System.out.println(rsmd.getColumnName(i)+" "+rsmd.getColumnType(i));
		}
	}

	// Print out the metadata for a search query
	void printMetaData(String query) throws SQLException {
		Statement stmt=con.createStatement();
		System.out.println(query);
		ResultSet rs = stmt.executeQuery(query);
		printMetaData(rs);
		rs.close();
		stmt.close();
	}
}
